@@ -14,13 +14,19 @@ TORCH_HIP_API void _record_memory_history(
     bool record_context = true,
     int64_t trace_alloc_max_entries = 1,
     bool trace_alloc_record_context = false,
-    bool record_cpp_context = false);
+    bool record_cpp_context = false,
+    bool clearHistory = false,
+    bool compileContext = false,
+    bool globalRecordAllocations = false);
 
 TORCH_HIP_API void _record_memory_history(
     std::optional<std::string> enabled = "all",
     std::optional<std::string> context = "all",
     const std::string& stacks = "all",
-    size_t max_entries = SIZE_MAX);
+    size_t max_entries = SIZE_MAX,
+    bool clearHistory = false,
+    bool compileContext = false,
+    bool globalRecordAllocations = false);
 
 TORCH_HIP_API std::string _memory_snapshot_pickled();
 
